import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import { cloneDeep } from 'lodash';
import FieldModelPublic from './public/FieldModelPublic';

var FieldModel = function FieldModel(_ref) {
  var component = _ref.component,
      name = _ref.name,
      value = _ref.value,
      validator = _ref.validator,
      reload = _ref.reload,
      excluded = _ref.excluded;

  _classCallCheck(this, FieldModel);

  _initialiseProps.call(this);

  if (!name) {
    throw new Error('Name is required');
  }
  this.component = component;
  this.name = name;
  this.excluded = !!excluded;
  this.initialValue = value;
  this.handleReload = reload || function () {};
  this.setValidator(validator);
  this.reset();
  this.submissionErrors = [];
  this['public'] = new FieldModelPublic(this);
};

var _initialiseProps = function _initialiseProps() {
  var _this = this;

  this.getPublic = function () {
    return _this['public'];
  };

  this.setValidator = function (validator) {
    _this.validator = validator || function () {
      return [];
    };
  };

  this.setValue = function (value) {
    _this.value = value;
  };

  this.setFocus = function (focused) {
    _this.focused = focused;
  };

  this.setTouched = function (touched) {
    _this.touched = touched;
  };

  this.setDirty = function (dirty) {
    _this.dirty = dirty;
  };

  this.validate = function () {
    _this.errors = [];
    _this.errors = _this.validator(_this.name, _this.value) || [];
  };

  this.setFormErrors = function (errors) {
    _this.formErrors = [];
    if (Array.isArray(errors)) {
      _this.formErrors = errors;
    } else if (errors) {
      _this.formErrors = [errors];
    }
  };

  this.setSubmissionErrors = function (errors) {
    _this.submissionErrors = [];
    if (Array.isArray(errors)) {
      _this.submissionErrors = errors;
    } else if (errors) {
      _this.submissionErrors = [errors];
    }
  };

  this.reload = function () {
    _this.handleReload();
  };

  this.reset = function () {
    _this.setValue(cloneDeep(_this.initialValue));
    _this.focused = false;
    _this.touched = false;
    _this.dirty = false;
    _this.submitted = false;
    _this.changedAfterSubmit = false;
    _this.errors = [];
    _this.formErrors = [];
  };

  this.makeDirty = function () {
    _this.touched = true;
    _this.dirty = true;
  };

  this.makeSubmitted = function () {
    _this.submitted = true;
    _this.changedAfterSubmit = false;
  };

  this.isValid = function () {
    var isValid = !_this.errors.length && !_this.formErrors.length;
    if (_this.submitted && !_this.changedAfterSubmit) {
      return isValid && !_this.submissionErrors.length;
    }
    return isValid;
  };

  this.isExcluded = function () {
    return _this.excluded;
  };

  this.getErrors = function () {
    var errors = [].concat(_this.errors, _this.formErrors);
    if (_this.submitted && !_this.changedAfterSubmit) {
      errors = [].concat(errors, _this.submissionErrors);
    }
    return errors;
  };
};

export default FieldModel;