import _Object$values from 'babel-runtime/core-js/object/values';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import FieldModel from './FieldModel';

var FieldsModel = function FieldsModel() {
  var _this = this;

  _classCallCheck(this, FieldsModel);

  this.get = function (name) {
    return _this.fields[name];
  };

  this.field = function (name) {
    return _this.fields[name];
  };

  this.add = function (props) {
    var field = new FieldModel(props);
    _this.fields[field.name] = field;
    return field;
  };

  this.remove = function (name) {
    delete _this.fields[name];
  };

  this.validate = function (name) {
    if (name) {
      _this.fields[name].validate();
    } else {
      _this.list().forEach(function (field) {
        return field.validate();
      });
    }
  };

  this.addErrors = function (errors) {
    _this.list().forEach(function (field) {
      var error = errors[field.name];
      field.setFormErrors(error);
    });
  };

  this.reload = function () {
    _this.list().forEach(function (field) {
      return field.reload();
    });
  };

  this.list = function () {
    return _Object$values(_this.fields);
  };

  this.data = function () {
    var data = {};
    _this.list().forEach(function (field) {
      return data[field.name] = field.value;
    });
    return data;
  };

  this.reset = function (name) {
    if (name) {
      _this.fields[name].reset();
    } else {
      _this.list().forEach(function (field) {
        return field.reset();
      });
    }
  };

  this.init = function () {
    var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _this.list().forEach(function (field) {
      return field.setValue(data[field.name]);
    });
  };

  this.makeDirty = function () {
    _this.list().forEach(function (field) {
      return field.makeDirty();
    });
  };

  this.makeSubmitted = function () {
    _this.list().forEach(function (field) {
      return field.makeSubmitted();
    });
  };

  this.isValid = function () {
    return _this.list().every(function (field) {
      return field.isValid();
    });
  };

  this.isTouched = function () {
    return _this.list().some(function (field) {
      return field.touched;
    });
  };

  this.values = function () {
    var values = {};
    _this.list().filter(function (field) {
      return !field.isExcluded();
    }).forEach(function (field) {
      return values[field.name] = field.value;
    });
    return values;
  };

  this.errors = function () {
    var errors = {};
    _this.list().forEach(function (field) {
      return errors[field.name] = field.getErrors();
    });
    return errors;
  };

  this.getPublic = function () {
    var fields = {};
    _this.list().forEach(function (field) {
      return fields[field.name] = field.getPublic();
    });
    return fields;
  };

  this.fields = {};
};

export default FieldsModel;